#!/usr/bin/env python3
#coding=utf-8
import numpy as np
import json
import sklearn
from sklearn.linear_model import SGDClassifier
from sklearn.multiclass  import OneVsRestClassifier
from gensim.models import FastText
#from Career_Platform.parser.database import backend_database_connection
#from Career_Platform.parser.exceptions import TrainDataException
import re,os,pickle

class LabelClassifier:
    """
    base class for label classifier
    --------
    Parameter:
        label_dict: a map from integer to string label
    """
    def __init__(self,label_dict_path='./config/labels.txt'):
        self.labeldict = {}  # dictionary that maps int to label text
        self.labeldictR = {} # maps label text to int
        self.load_label_dict(label_dict_path)
        #print(self.labeldictR)

    def load_label_dict(self,filename):
        with open(filename,encoding='UTF-8') as f:
            data = f.read().splitlines()
        self.labeldict = { i:a.split('\t')[0] for (i,a) in enumerate(data)}
        self.labeldictR = { a:i  for (i,a) in self.labeldict.items()}

    '''
    def update_classifier_db(self,list_rid,list_uid,list_mseg,list_mlabel):
        for index,thing in  enumerate(list_mseg):
            list_mseg[index]=list_mseg[index].replace('，','').replace('|','').replace(' ','').replace(',','')

        print(list_mlabel)
        self.train_update(list_mseg, list_mlabel)

        # update the database with new prediction
        labels = [ ','.join(self.classify(sen) ) for sen in list_mseg ]
        j = backend_database_connection()
        j.save_db(labels,list_uid,list_rid)
    '''
    # abstract function to be reimplemented in subclasses.
    def classify(self,string):
        pass
    # abstract function to be reimplemented in subclasses.
    def train_update(self, list_mseg, list_mlabel):
        pass

class ManualLabelClassifier(LabelClassifier):
    """ a rule-based  classifier using keywords
        can be accelerated using a reversed dictionary or
        suffix tree string matching """
    def __init__(self, rules={}):
        LabelClassifier.__init__(self)
        self.rule_dict = rules

    def classify(self,string):
        labels=[]
        for (label,keywords) in  self.rule_dict.items():
            for word in keywords:
                if(word in string):
                    labels.append(label)
                    break
        if(len(labels)==0):
            labels.append('undefined')
        return labels

    def import_label_rules(self,filename):
        with open(filename,encoding='UTF-8') as f:
            data = f.read().splitlines()
            f.close()

        self.rule_dict = {}
        for line in data:
            words =line.split('\t')
            if len(words)>1 and words[1]!='':
                self.rule_dict[words[0]] = words[1].split(',')
            else:
                self.rule_dict[words[0]] = []

class ExpRuleClassifier(LabelClassifier):
    def __init__(self,rule_label_path = './config/rule_label.json',hybrid_label_path = './config/hybrid_rule_label.json'):
        LabelClassifier.__init__(self)
        f = open(rule_label_path,encoding='utf-8')
        self.label_dic = json.load(f)
        f.close()
        self.groupkeys = list(self.label_dic.keys())
        f = open(hybrid_label_path,encoding='utf-8')
        self.hylabel_dic = json.load(f)
        f.close()
        self.hybridkeys = list(self.hylabel_dic.keys())
        
    def classify(self,text,Hybrid = True):
        final = {}
        for groupname in self.groupkeys:
            res = self.groupclassify(groupname,text)
            final = {**final,**res}
        if Hybrid:
            final = self.addhybridlabel(final)
        return final
    
    def groupclassify(self,groupname,text,f=1):
        jsondict = self.label_dic
        group = jsondict[groupname]
        innerkeys = list(group.keys())
        addition = innerkeys[3:]
        
        result = {group['default']:False}
        for label in addition:
            result[label] = False
        
        if f==1:
            f = self.method1
        elif f==2:
            f = self.method2
        flag = f(text,group['isexist'])
        if f(text,group['remove']):
            flag = False
        if flag:
            for label in addition:
                temp = group[label]
                if f(text,temp):
                    result[label] = True
                    if result.get('')!=None:
                        del result['']
                    return result
            result[group['default']] = True
        if result.get('')!=None:
            del result['']
        return result
    
    def hybridclassify(self,labelname,D):
        jsondict = self.hylabel_dic
        f = eval(jsondict[labelname])        
        result = {labelname:False}        
        result[labelname] = f(D)
        return result
    
    def addhybridlabel(self,result_dict):
        for labelname in self.hybridkeys:
            res = self.hybridclassify(labelname,result_dict)
            result_dict = {**result_dict, **res}
        return result_dict
    '''存在性判断'''
    def method1(self,text,D):
        for item in D['keyword']:
            if item in text:
                return True
        for reg in D['regular']:
            if re.search(reg,text) != None:
                return True
        return False
    '''任意性判断'''
    def method2(self,text,D):
        for item in D['keyword']:
            if item not in text:
                return False
        return True
    '''简单的Hybrid Label生成法'''
    def method3(self,text,L,n=2):
        count = 0
        for item in L:
            if item in text:
                count += 1
                if count>=n:
                    return True
        return False
        
    
class ExpKnowClassifier(LabelClassifier):
    def __init__(self,excelpath = None):
        LabelClassifier.__init__(self)
        
class OneVsRestSGDClassifier(LabelClassifier):

    def __init__(self,f_dim=100,ft_iters=20,update_iters=100,
                 label_dict_path='data/labels.txt'):

        LabelClassifier.__init__(self,label_dict_path)
        self.f_dim = f_dim # dimension of word feature vector
        self.ft_iters = ft_iters
        self.update_iters =update_iters

        self.ft_model = FastText(min_count=1, size=self.f_dim)
        self.clf = OneVsRestClassifier( SGDClassifier(loss='modified_huber' ,
                                      class_weight={0:0.4,1:0.6},
                                      penalty='l2' ,warm_start=False,
                                      random_state=1))

    def init_fasttext(self,model_path=None,train_data=None ):
        """
        if train_data is provided, train a new fasttext model;
        otherwise, load it from the given path

        --------
        Parameter:

            model_path: fasttext model prefix

            train_data: a list of tokenized sentences. if not provided,
                will try to load existing model from model_path

        """

        if not train_data and model_path and os.path.isfile(model_path):
            #=== load exisitng model ====
            print('loading fasttext model from',model_path)
            self.ft_model = FastText.load(model_path)

        elif train_data:
            #=== train fast text model ====
            # if train_data is not a list of list, split each sentence
            # into list of words
            print('training fasttext model from scratch...' )
            train_data = [re.split(',| ',r) if (not isinstance(r,list)) else r\
                          for  r in  train_data ]

            self.ft_model.build_vocab(train_data)
            self.ft_model.train(train_data, total_examples=len(train_data) ,
                               epochs = self.ft_iters)
            if model_path:
                self.ft_model.save(model_path,separately=[])
        else:
            #=== no train data and no model path provided
            raise TrainDataException('Error building fasttext model. No data/model provided.')


    def div_norm(self,x):
        norm_value = np.sqrt(np.sum(x**2)) #l2norm
        if norm_value > 0:
            return x * ( 1.0 / norm_value)
        else:
            return x

    def sentence_to_vec(self, words):
        """ generating embedding by summing up normalized
        word embeddings

        --------
        Parameter:
            words: a list of words or a string representation of a sentence
            (seperated by space or ',' )

        Return:
            sentence embedding matrix of size len(words) x f_dim

        """
        if  not isinstance( words ,list):
            words =  re.split(',| ', words  )

        vecs = np.zeros( (len(words),self.f_dim) )
        for i,word in enumerate(words):
            v = self.ft_model.wv.get_vector(word)
            vecs[i] = self.div_norm(v)
        return np.mean(vecs,axis=0)

    def to_vec(self,data):
        """ batch computation of sentence embeddings """
        vec = np.zeros( (len(data),self.f_dim))
        for i,sentence in enumerate(data):
            vec[i] = self.sentence_to_vec(sentence)

        return vec


    def train(self, train_data, train_label):
        """
        offline training of the SGD classifier

        --------
        Parameters:

            train_data: a list of tokenized sentences. Each sentence is either
                a string deliminated by comma or space, or a list of words.

            train_label: a list of labels. Each label is a string deliminated
                by comma or space.
        Return:

            X: sentence embedding matrix of size len(train_data) x f_dim
            Y: binary label matrix of size len(train_data) x #_classes
        """
        print('training multilabel classifier on %d samples...' % len(train_data))
        Y = np.zeros((len(train_label), len(self.labeldict) ) )
        for i, labels in enumerate(train_label):
            label_list = re.split(',| ',labels)

            for l in label_list:
                if l:
	                Y[i,self.labeldictR[l]]=1

        # add dummy sample to classes that do not have samples
        indices = np.where(np.sum(Y,axis=0)==0)[0]
        Y_new = np.zeros( (len(indices),Y.shape[1] ))
        for i,id in enumerate(indices):
            train_data.append([self.labeldict[id]] )
            Y_new[i,id]=1
        Y=np.vstack((Y, Y_new ) )



        X = self.to_vec(train_data)
        self.clf.fit(X,Y)
        return X,Y



    def train_update(self, train_data, train_label ):
        """
        online training of the SGD classifier

        --------
        Parameters: see train()

        """
        Y = np.zeros((len(train_label), len(self.labeldict) ) )
        X = self.to_vec(train_data)
        for i, labels in enumerate(train_label):
            label_list = re.split(',| ',labels)
            for l in label_list:
                if l :
                    Y[i,self.labeldictR[l]]=1
        for i in range(self.update_iters):
            self.clf.partial_fit(X, Y )
        return X,Y

    def classify(self,string):
        """
        predict the labels of a tokenized sentence

        --------
        Parameters:
            string: string delimited by comma or space, or a list of words

        Return:
            labels: a list of predicted labels

        """
        X = self.to_vec( [string ] )
        Y = self.clf.predict( X)
        #print('class probability',self.clf.predict_proba(X) )
        labels = [ self.labeldict[id] for id in np.nonzero(Y[0])[0] ]


        return labels

    def save_clf(self,filename):
        print('writing classification model to',filename,'...')
        with open(filename,'wb') as f:
            pickle.dump(self.clf ,f)

    def load_clf(self,filename):
        print('loading classification model from',filename,'...')
        with open(filename,'rb') as f:
            self.clf = pickle.load(f)

class WordMoverKNNClassifier(LabelClassifier):
    """
    a knn classifier based on word mover distance between sentences

    """
    pass

if __name__ == '__main__':
    import time
    text1 = '清华大学深圳研究院电子通信硕士'
    text2 = '深圳市罗湖区街道派出所民警'
    text3 = '深圳市交通运输委员会福田交通运输局副局长'

    t0 = time.time()
    label = ExpRuleClassifier()
    print('Obj building Consuming:',time.time()-t0,'s')

    print('\nSample Text:',text1)
    t0 = time.time()
    print(label.classify(text1,True))
    print('Time Consuming:',time.time()-t0,'s')

    print('\nSample Text:',text2)
    t1 = time.time()
    print(label.classify(text2,True))
    print('Time Consuming:',time.time()-t1,'s')

    print('\nSample Text:',text3)
    t1 = time.time()
    print(label.classify(text3,True))
    print('Time Consuming:',time.time()-t1,'s')


#!/usr/bin/env python3
#coding=utf-8
import re
# from Career_Platform.octree.OCtree import CTree

class Experience:
    def __init__(self,text, time=None, loc=None, org=None,seg=None,labels=None):
        self.text=text
        self.time=time
        self.organization=org
        self.location=loc
        self.segmented = seg # segmented string
        self.labels = labels # list of labels
        self.batch_id =None
class WorkExperience(Experience):
    def __init__(self, text, time=None, loc=None, org=None,seg=None,
                 labels=None, pos=None):
        Experience.__init__(self,text,time,loc,org,seg,labels)
        self.position = pos

class Person:
    def __init__(self,name):
        self.name = name
        self.work_exp = []  #indexed by rid


class Catalog:
    def __init__(self):
        self.users = [] # indexed by uid
        self.tokenizer = None
        self.classifier = None

    def clear(self):
        self.users = []

    def print_2(self, raw=False,max_n = 2000):
        ct = 0
        for u in self.users:
            print('name:', u.name )
            for i,exp in enumerate(u.work_exp):
                if ct >= max_n:
                    return
                if raw:
                    print(i,exp.text)
                else:
                    print(i,exp.time,'parsed:',exp.segmented,
                          'labels:',exp.labels, 'loc:',exp.location,
                          'pos:',exp.position,'batch:',exp.batch_id)
                ct+=1


    def get_all_parsed(self):
        result=[]
        for u in self.users:
            for exp in u.work_exp:
                result.append(exp.segmented)
        return result

    def get_all_labels(self):
        result=[]
        for u in self.users:
            for exp in u.work_exp:
                result.append(exp.labels)
        return result

# This function is defined to classify the entries into some general catafories
    # To use this function, you must clarify which parameter you use. 'write_file' or 'read_file'
    # If you use write_file parameter, remember to add parameter 'result'. 
    # The 'result' usually comes from self.parse_catalog() function
    # Otherwise, unexpected error may rise
    def get_classified_parsed(self,result=None,write_file = None,read_file = None,):
        if (read_file == None) and (write_file != None):
            flag = [0]*len(result)
            
            for i in range(len(result)):
                if flag[i] == 1:
                    print('skip %d'%i)
                    continue

                print(result[i])

                while True:
                    if len(result[i]) <= 6 or result[i].find(' ') == -1:
                        key_word = result[i]
                        label = 7
                        break
                    else:
                        key_word = input('input a keyword:\t')
                        try:
                            label = int(input('input the classification:\t'))
                        except:
                            continue

                    if self.key_word_in_entry(result[i],key_word) and (label<=7 and label>=1) :
                        break
                
                flag[i] = 1
                index = result[i].index(key_word)
                
                
                for j in range(i+1,len(result)):
                    if self.key_word_match(result[j][:index+len(key_word)],result[i][:index+len(key_word)]):
                        result[j] = self.manual_classifier(result[j],label)
                        flag[j] = 1
                    else:
                        continue
                
                result[i] = self.manual_classifier(result[i],label)

            with open(write_file,'w',encoding='utf-8') as f:
                for entry in result:
                    f.write(entry+'\n')
            return result

        elif (read_file != None) and (write_file == None):
            result = []
            with open(read_file,'r',encoding = 'utf-8') as f:
                for line in f:
                    result.append(line[:-1])
            print(result)
            return result

        else:
            print('Error! You must specify only one parameter! You will choose either read_file or write_file.')
            print('If you choose read_file, you use the data txt that has been classified manually before.')
            print('If you choose write_file, you will be classifying the entries on site manually. It might be time-consuming!')
            print('If this is your first time to use this function, you need to use write_file parameter. Otherwise,\
                I suggest you use \'read_file\' parameter to save your time')
    
    def key_word_match(self,result,match):
        return result == match

    def key_word_in_entry(self,result,key_word):
        return str(key_word+' ') in result or str(' '+key_word+' ') in result or str(' '+key_word) in result
    
    def manual_classifier(self,result,label):
        if label == 1:
            result = '市政协直属 '+result
        elif label == 2:
            result = '市人大直属 '+result
        elif label == 3:
            result = '市委直属 '+result
        elif label == 4:
            result = '市政府直属 '+result
        elif label == 5:
            result = '事业单位/国企 '+result
        elif label == 6:
            result = '军检法机构 '+result
        elif label == 7:
            result = '其他 '+result
        return result



    def add_tokenizer(self, tokenizer):
        self.tokenizer = tokenizer


    def parse_catalog(self,HMM=True):
        if not self.tokenizer:
            print('Error: Please add a tokenizer first!')
            return
        for user in self.users:
            for exp in user.work_exp:
                result = self.tokenizer.parse_strings([exp.text],simple=False,HMM=HMM)[0]
                if len(result)<=3:
                    print('[Error] fail to parse', exp.text)
                    continue
                result = result[3:]
                exp.location=''
                org_pos = []
                exp.segmented=' '.join([word for  word,flag in result ])
                for word,flag in result:
                    if flag == 'ns' or flag == 'LOC':
                        exp.location += word

                    if flag != 'm' and flag != 'x':
                        org_pos.append( (word,flag) )
                if len(org_pos)>1:
                    exp.position =   org_pos[-1][0]
                    exp.organization=''.join( [w for w,f in org_pos[:-1]])

    def add_classifier(self,cla):
        self.classifier = cla

    def classify_all(self):
        if not self.classifier:
            print('Error: Please add a label classifier first!')
        for user in self.users:
            for exp in user.work_exp:
                print(exp.text)
                if exp.organization and exp.position:
                    exp.labels  = self.classifier.classify(exp.organization+exp.position)
                else:
                    exp.labels = ['undefined']


class ResumeFileIO(Catalog):
    def __init__(self):
        Catalog.__init__(self)
        #self.print_2(raw=True)


    # This function is defined to write each entry of the user data to one line,
    # rather than break into several lines as in ori.txt,
    # which is quite helpful for comma-removing later
    def pre_user_data(self,file_path):
        f_ori = open(file_path,'r',encoding='utf-8')
        entry_list = []
        new_entry_list = []
        i = 0

        for line in f_ori:
            line = line.replace(',','，')
            line = re.sub("[；|;|。].*$","",line)
            line = line.strip()
            line = self.remove_parentheses(line)

            # if we use "-今" instead of "-今     ", unexpected error happened.
            # So i replace it with "-今     "
            line = re.sub("—今     ","—9999.99",line)
            entry_list.append(line)

        while i <= len(entry_list)-1:
            # set a convergent condition
            if i == len(entry_list)-1:
                new_entry_list.append(entry_list[i])
                break

            else:
                new_entry = entry_list[i]
                # cond_1 means next line does not start with '---'
                cond_1 = ( re.match('^-.*',entry_list[i+1]) == None )
                # cond_2 means next line does not start with number 1 to 9
                cond_2 = ( re.match('^[1-9].*',entry_list[i+1]) ==None )

                while cond_1 and cond_2 :
                    i = i+1
                    new_entry += entry_list[i]
                    if i < len(entry_list)-1:
                        cond_1 = ( re.match('^-.*',entry_list[i+1]) == None )
                        cond_2 = ( re.match('^[1-9].*',entry_list[i+1]) == None )
                    else:
                        break
                new_entry_list.append(new_entry)
                i += 1
        return new_entry_list

    def remove_comma_round_1(self,pre_data,file_path):
        # remove comma_2, do not create entries, and write into file
        list_with_comma_2 = []

        # 此函数用于替换字符串中特定位置的字符
        def replace_char(string,char,index):
            string = list(string)
            string[index] = char
            return ''.join(string)

        for entry in pre_data:
            #去掉job_experience开头的“兼”或“兼职"字眼
            blank_ind = entry.find(' ')
            job = entry[blank_ind+1:]
            if job[0] == '兼':
                entry = entry[:blank_ind+1] + job[1:]
            if job[:2] == '兼职':
                entry = entry[:blank_ind+1] + job[2:]

            # 找到“兼”的index,计算其到下一个标点（若有）的字符串长度，
            # 和阈值（暂定为8）比较，小于等于阈值替换为“、” ，大于阈值替换为“，”
            threshold = 8
            tmp_entry = ''
            part_time_ind = entry.find('兼')
            while part_time_ind != -1:
                tmp_entry = tmp_entry[part_time_ind+1:]
                tmp_index_1 = tmp_entry.find('，')
                tmp_index_2 = tmp_entry.find('、')
                if (tmp_index_1 != -1) and (tmp_index_2 != -1):
                    tmp_index = min(tmp_index_1,tmp_index_2)
                if (tmp_index_1 == -1) and (tmp_index_2 != -1):
                    tmp_index = tmp_index_2
                if (tmp_index_1 != -1) and (tmp_index_2 == -1):
                    tmp_index = tmp_index_1
                else:
                    # 当后面没有任何标点时,设tmp_index = 1000,取整个entry
                    tmp_index = 1000

                part_time_entry = entry[part_time_ind+1:tmp_index]
                if len(part_time_entry) > threshold:
                    entry = replace_char(entry,'，',part_time_ind)
                else:
                    entry = replace_char(entry,'、',part_time_ind)
                part_time_ind = tmp_entry.find('兼')

            entry = entry.replace('，，','，')
            entry = entry.replace('，、','、')
            entry = entry.replace('、、','、')
            entry = entry.replace('、，','，')

            comma_ind = entry.find('，')
            while comma_ind != -1:
                blank_ind = entry.find(' ')
                list_with_comma_2.append(entry[:comma_ind])
                entry = entry[:blank_ind]+'  '+entry[comma_ind+1:]
                comma_ind = entry.find('，')
            list_with_comma_2.append(entry)

        with open(file_path,'w',encoding='utf-8') as f:
            for entry in list_with_comma_2:
                comma_2_ind = entry.find('、')
                if comma_2_ind != -1:
                    f.write(entry[:comma_2_ind]+'\n')
                else:
                    f.write(entry+'\n')
        f.close

    def remove_comma_round_2(self,pre_data,tree,output_path):
        # remove comma_2, create new entries via prefix, and write into file
        list_with_comma_2 = []

        # 此函数用于替换字符串中特定位置的字符
        def replace_char(string,char,index):
            string = list(string)
            string[index] = char
            return ''.join(string)

        for entry in pre_data:
            #去掉entry开头的“兼”或“兼职"字眼
            blank_ind = entry.find(' ')
            job = entry[blank_ind+1:]
            if job[0] == '兼':
                entry = entry[:blank_ind+1] + job[1:]
            if job[:2] == '兼职':
                entry = entry[:blank_ind+1] + job[2:]

            # 找到“兼”的index,计算其到下一个标点（若有）的字符串长度，
            # 和阈值（暂定为8）比较，小于等于阈值替换为“、” ，大于阈值替换为“，”
            threshold = 8
            tmp_entry = ''
            part_time_ind = entry.find('兼')
            while part_time_ind != -1:
                tmp_entry = tmp_entry[part_time_ind+1:]
                tmp_index_1 = tmp_entry.find('，')
                tmp_index_2 = tmp_entry.find('、')
                if (tmp_index_1 != -1) and (tmp_index_2 != -1):
                    tmp_index = min(tmp_index_1,tmp_index_2)
                if (tmp_index_1 == -1) and (tmp_index_2 != -1):
                    tmp_index = tmp_index_2
                if (tmp_index_1 != -1) and (tmp_index_2 == -1):
                    tmp_index = tmp_index_1
                else:
                    tmp_index = 1000

                part_time_entry = entry[part_time_ind+1:tmp_index]
                if len(part_time_entry) > threshold:
                    entry = replace_char(entry,'，',part_time_ind)
                else:
                    entry = replace_char(entry,'、',part_time_ind)
                part_time_ind = tmp_entry.find('兼')

            entry = entry.replace('，，','，')
            entry = entry.replace('，、','、')
            entry = entry.replace('、、','、')
            entry = entry.replace('、，','，')
            comma_ind = entry.find('，')
            while comma_ind != -1:
                blank_ind = entry.find(' ')
                list_with_comma_2.append(entry[:comma_ind])
                entry = entry[:blank_ind]+'  '+entry[comma_ind+1:]
                comma_ind = entry.find('，')
            list_with_comma_2.append(entry)

        with open(output_path,'w',encoding='utf-8') as f:
            b = []
            for entry in list_with_comma_2:
                comma_2_ind = entry.find('、')
                blank_ind = entry.find(' ')
                time_period = entry[:blank_ind]
                if comma_2_ind == -1:
                    f.write(entry+'\n')

                else:
                    a = entry[blank_ind+2:comma_2_ind]
                    n1,_ = tree.greedy_prefix_match(a)
                    prefix_list= [ n.data.name for n in tree.get_prefix(n1)]
                    prefix = ''

                    for i in range( max( (len(prefix_list)-1), 1 )):
                        prefix += prefix_list[i]

                    flag = True
                    while comma_2_ind != -1:
                        if flag == True:
                            f.write(entry[:comma_2_ind]+'\n')
                            flag = False
                        else:
                            entry = entry[comma_2_ind+1:]
                            comma_2_ind = entry.find('、')
                            if comma_2_ind == -1:
                                f.write(time_period + '  ' + prefix + entry+'\n')
                                break
                            else:
                                f.write(time_period + '  ' + prefix + entry[:comma_2_ind]+'\n')
        f.close()

    def time_filter(self,
                year,  # only support for one year filter.
                       # e.g, input "2012", it will show all the entries incluing 2012.
                input_file_path):
        with open(input_file_path,'r',encoding='utf-8') as in_file:
            list1 = in_file.readlines()

        output_file_path = 'data/%s_period.txt'%str(year)

        with open(output_file_path,'w',encoding='utf-8') as out_file:
            for line in list1:
                if line.startswith('-') or line.endswith('xls\n'):
                    out_file.write(line)
                else:
                    dash_ind = line.find('—')
                    blank_ind = (' ')
                    t_period_start = line[:4]
                    t_period_end = line[dash_ind+1:dash_ind+5]
                    if year >= int(t_period_start) and year <= int(t_period_end) :
                        out_file.write(line)


    def read_resume_txt(self,file_path):
        newPerson = None
        newEntry = None
        with open(file_path,'r',encoding='UTF-8' ) as f:
            for line in f:
                line= line.strip()
                m_head = re.match("([0-9]+).xls",line)
                m_entry = re.match("(\d{4}.\d{1,2}?—\d{4}.\d{1,2})\s*(.+)",line)
                # m_entry_cont = re.match("^(?![-|—]).*",line)
                if m_head :
                    if newPerson:
                        if newEntry:
                            newPerson.work_exp.append(newEntry)
                            newEntry=None
                        self.users.append(newPerson)
                    newPerson = Person(m_head.group(1))
                    #print(newPerson.name)
                elif m_entry:
                    if newEntry and len(newEntry.text)>=20 :
                        # only add entries at least 20 characters long
                        newPerson.work_exp.append(newEntry)
                    time = m_entry.group(1)
                    text = m_entry.group(2)
                    newEntry = WorkExperience(line,time=time )
                    # print(newEntry.text)
                # elif m_entry_cont:
                   #  newEntry.text +=line


    def remove_parentheses(self,line):
        p_left_index = line.find('(')
        if p_left_index == -1:
            pass
        else:
            p_right_index = line.find(')')
            content = line[p_left_index+1:p_right_index]
            content = content.strip()
            if content =='':
                line = line[:p_left_index]+line[p_right_index+1:]
            else:
                line = line[:p_left_index]+content+line[p_right_index+1:]

        return line

        # return re.sub("\(.*\)","",line)

   # def remove_text_after_comma(self,line):
        #return re.sub("[、|，|；|,|;].*$","",line)



from labeler.label_classifier import ExpRuleClassifier
import os
import time

#print('hello, here is:',os.getcwd())

text1 = '清华大学深圳研究院电子通信硕士'
text2 = '深圳市罗湖区街道派出所民警'
text3 = '深圳市交通运输委员会福田交通运输局副局长'

t0 = time.time()
label = ExpRuleClassifier()
print('Obj building Consuming:',time.time()-t0,'s')

print('\nSample Text:',text1)
t0 = time.time()
print(label.classify(text1,True))
print('Time Consuming:',time.time()-t0,'s')

print('\nSample Text:',text2)
t1 = time.time()
print(label.classify(text2,True))
print('Time Consuming:',time.time()-t1,'s')

print('\nSample Text:',text3)
t1 = time.time()
print(label.classify(text3,True))
print('Time Consuming:',time.time()-t1,'s')

# Testing on a mini dataset.
f = open('final_version.txt','r',encoding='utf-8')
data = f.readlines()
f.close()
data = [item.strip().split()[1] for item in data if ' ' in item]

clf = ExpRuleClassifier()
toshow = []
count = 0
print('\nTesting on a mini dataset.')
t0 = time.time()
for resume in data:
    res_dict = clf.classify(resume)
    predict = [label for (label,value) in res_dict.items() if value==True]
    '''
    if '深圳' in predict:
        predict.remove('深圳')'''
    if predict == []:
        toshow.append('0 '+resume)
    else:
        toshow.append('1 '+resume+'  '+str(predict))
        count += 1
print('Time Consuming:',time.time()-t0,'s')
print('Labeled resume:',count,'Total resume:',len(toshow))


#!/usr/bin/env python3
#coding=utf-8

from __future__ import print_function, unicode_literals

import jieba
import jieba.posseg as pseg
import re
from Career_Platform.parser.db_interface import DBInterface
from pdb import set_trace as bp
class Tokenizer:
    def __init__(self,db_options):

        self.db = DBInterface(db_options)

    def parse_strings(self,list_of_txt):
        pass


    def update_dict_db(self, list_mseg,multiplier=1,
                        remove_containing_words=False,ignore_singleton=True):
        """ add words in list_mseg to tokenizer and the word database
        -------------
        Parameters:
            list_mseg: list of user segmented texts
        """
        # convert list of sentence to a list of list
        list_mseg = [re.split(',| ',r) if (not isinstance(r,list)) \
                            else r for  r in  list_mseg ]
        # flatten to a single list
        words = [ w for  sentence in list_mseg for w in sentence]
        if ignore_singleton:
            # do not add words of single character
            words = [ w for  sentence in list_mseg for w in sentence if len(w)>1]
        # remove from tokenizer prefix words
        new_words = {w:words.count(w) for w in  words }
        if remove_containing_words:
            to_remove= self.db.remove_containing_words_from_db(new_words.keys())
            for w in to_remove:
                self.remove_user_word(w)

        # add to database
        print(new_words)
        self.db.add_words_to_db(new_words )
        #update tokenizer with new words
        for w,freq  in new_words.items():
            self.add_user_word(w,freq = freq*multiplier)
        return list_mseg


    def update_tokenizer_db(self,list_uid,list_rid,list_mseg):
        """ add words in list_mseg to tokenizer and the word database
        -------------
        Parameters:
            list_uid: list of user ids
            list_rid: list of resume ids
            list_mseg: list of user segmented texts
        """
        # convert list of sentence to a list of list
        list_mseg = self.update_dict_db(list_mseg)
        # put manual seg result in database
        for i in range(len(list_uid)):
            self.db.set_user_data_by_id(list_uid[i],list_rid[i],'mseg',
                                        ' '.join(list_mseg[i]))

    def save_userdict(self,filename,mode='w'):
        """ save words in db into a text file """
        print('saving user dict words to',filename)
        word_entries= self.db.read_words_from_db( )
        with open(filename,mode,encoding='UTF-8') as f:
            lines = ['%s %d %s\n' % (word,freq,part) for word,part,freq in word_entries]
            f.writelines(lines)
            f.close()


    def load_userdict_from_txt(self,filename,sync_db=True):
        """ load userdict from text file """
        print('load user dict',filename,'... sync database...')
        # sync database table with what's in txt files
        #self.db.initialize_words_db()

        # load txt file to the tokenizer
        self.load_userdict(filename)
        if sync_db:

            with open(filename,'r' ) as f:
                lines = f.read().splitlines()
            data =[]
            for i, str in enumerate(lines):
                entry = str.split(' ')
                part = entry[2] if len(entry)==3 else ''

                data.append((i, entry[0], part, int(entry[1])))

            self.db.write_word_dict_to_db(data)



    def load_userdict_from_db(self):
        """ load userdict from word table in the database """
        word_entries = self.db.read_words_from_db( )
        for (word, part ,freq) in word_entries:
            self.add_user_word(word,freq)
        print('loaded',len(word_entries),
              'words from userword database')





class JiebaTokenizer(Tokenizer):
    """
    This class is used for Parsing
    Currently Using Jieba
    """
    def __init__(self,db_options):
        Tokenizer.__init__(self,db_options)
        #jieba.enable_paddle()


    def parse_strings(self,list_of_txt, simple=True,HMM=True):

        results = []

        for txt in list_of_txt:

            words = pseg.cut(txt,use_paddle=False, HMM=HMM)
            if simple:
                words = [ w for w ,flag in words if w != ' ']
            else:
                words = [(w ,flag) for w ,flag in words if w != ' ']

            results.append(words)
        return results

    def add_user_word(self,word,freq=50):
        print('adding',word,'freq=',freq, '..\n')
        jieba.add_word(word ,freq)

    def add_user_segs(self,list_of_words,multiplier=2):
        word_freq = {x:list_of_words.count(x) for x in  list_of_words }
        for (word,freq ) in word_freq.items():
            print('word=%s,freq=%d' % (word,freq))
            jieba.add_word(word,freq*multiplier)

    def remove_user_word(self,word):
        print('removing',word,'..\n')
        jieba.del_word(word)

    def suggest_seg(self,seg):
        """ suggest a particular segmentation """
        print('adding',seg,'..\n')
        jieba.suggest_freq(seg,True)

    def load_userdict(self,filename):
        """ load user dict from a file """
        jieba.load_userdict(filename)


    def _word_freq(self,w):
        """ a debug function to check the internal dictionary
        frequency of word w """

        f=jieba.get_FREQ
        return f(w)
'''
class HybridTokenizer(Tokenizer):
    """
    This class is used for Parsing
    Currently Using Jieba
    """
    def __init__(self):
        Tokenizer.__init__(self)

    def parse_strings(self,list_of_txt):
        pass

'''



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: liuzw
# Mail: helloliuzw@163.com
# Created Time:  2020-8-11 17:35:34
#############################################

from setuptools import setup, find_packages

setup(
    name = "SZlabeler",
    version = "0.0.3",
    keywords = ["pip", "shorttext","rule","career"],
    description = "A rule-based toolkit for resume classification",
    long_description = "A rule-based toolkit for resume classification, applied in ShenZhen",
    license = "MIT Licence",

    url = "https://github.com/helloliuzw/parser",
    author = "liuzw",
    author_email = "helloliuzw@163.com",

    packages = ['SZlabeler'],
    include_package_data = True,
    platforms = "any",
    install_requires = []
)

